/*
 * The following code is released under the terms of the IBM
 * Common Public License v.5.
 */
#include <jni.h>
#include <jawt_md.h>
#include <windows.h>
#include "SWT_AWT.h"

#define JVM_FILE_NAME_SIZE 255
typedef jboolean (JNICALL *JAWT_GetAWT_t)(JNIEnv* env, JAWT* awt);

HINSTANCE g_hJawt;
JAWT_GetAWT_t g_pfnJAWT_GetAWT;

/*
 * This is necessary because the JVM does not load the jawt.dll library
 * when started through the invocation API.
 */
BOOL loadJawtLibrary()
{
	TCHAR tJvmFileName[JVM_FILE_NAME_SIZE + 1];
	TCHAR tJawtFileName[JVM_FILE_NAME_SIZE + 1];

	HINSTANCE hJawt = LoadLibrary("jawt");

	if (hJawt == NULL) {
		HMODULE hJvmModule = GetModuleHandle("jvm");
		GetModuleFileName(hJvmModule, tJvmFileName, JVM_FILE_NAME_SIZE);
		*strrchr(tJvmFileName, '\\') = '\0'; /* remove .exe file name */

		/* first look in the same directory as jvm.dll */
		sprintf(tJawtFileName, "%s\\jawt.dll", tJvmFileName);
		hJawt = LoadLibrary(tJawtFileName);

		/* if that fails, try the directory above the jvm.dll dir */
		if (hJawt == NULL) {
			sprintf(tJawtFileName, "%s\\..\\jawt.dll", tJvmFileName);
			hJawt = LoadLibrary(tJawtFileName);
			if (hJawt == NULL) {
				return FALSE;
			}
		}
	}

	g_pfnJAWT_GetAWT = (JAWT_GetAWT_t)GetProcAddress(hJawt, "_JAWT_GetAWT@8");

	return TRUE;
}

BOOL APIENTRY DllMain(HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
	if (g_hJawt == NULL || g_pfnJAWT_GetAWT == NULL) {
		return loadJawtLibrary();
	} else {
		return TRUE;
	}
}

void throwSwtException(JNIEnv* env, char* message) {
	jclass jcSwtExceptionClass = (*env)->FindClass(env, "org/eclipse/swt/SWTException");

	/* Unable to find the new exception class, give up. */
	if (jcSwtExceptionClass == 0) {
		return;
	}

	(*env)->ThrowNew(env, jcSwtExceptionClass, message);
}

JNIEXPORT jint JNICALL Java_org_eclipse_swt_internal_awt_win32_SWT_1AWT_getHwndForCanvas
		(JNIEnv* env, jclass clazz, jobject canvas) {
	JAWT awt;
    JAWT_DrawingSurface* ds;
    JAWT_DrawingSurfaceInfo* dsi;
	JAWT_Win32DrawingSurfaceInfo* w32Dsi;
    jint lock;
	jint hwnd;

	/* Get the JAWT interface */
    awt.version = JAWT_VERSION_1_3;
    if (g_pfnJAWT_GetAWT(env, &awt) == JNI_FALSE) {
        throwSwtException(env, "JAWT not found");
		return -1;
    }

	/* Get the drawing surface */
    ds = awt.GetDrawingSurface(env, canvas);
    if (ds == NULL) {
        throwSwtException(env, "NULL drawing surface");
        return -1;
    }

    /* Lock the drawing surface */
    lock = ds->Lock(ds);
    if((lock & JAWT_LOCK_ERROR) != 0) {
        awt.FreeDrawingSurface(ds);
        throwSwtException(env, "Error locking surface");
        return -1;
    }

    /* Get the drawing surface info */
    dsi = ds->GetDrawingSurfaceInfo(ds);
    if (dsi == NULL) {
        ds->Unlock(ds);
        awt.FreeDrawingSurface(ds);
        throwSwtException(env, "Error getting surface info");
        return -1;
    }

	w32Dsi = (JAWT_Win32DrawingSurfaceInfo*)dsi->platformInfo;

	hwnd = (jint)w32Dsi->hwnd;

    /* Free the drawing surface info */
    ds->FreeDrawingSurfaceInfo(dsi);

    /* Unlock the drawing surface */
    ds->Unlock(ds);

    /* Free the drawing surface */
    awt.FreeDrawingSurface(ds);

	return hwnd;
}